# -*- coding: cp1252 -*-
#!/usr/bin/env  python
# ===================================
# by Ph. MARON, 3th February 2012
# =================================== 



from numpy import array, ndarray, float32, zeros
#==========================================================================================================================
class  Points  :
# ===================================
# by Ph. MARON, 6th december 2009
# =================================== 
    """ 
    Define a class of object associated with a Points file .xyz of MOHID code 
        .listpoints returns the list of coordinates of points inside the file 
            as [[x0,y0,z0],...,[xi,yi,zi],...,[xn,yn,zn]] 
        .xpoints returns the list of first coordinates of points inside the file 
            as [x0,...,xi,,..., xn] 
        .ypoints returns the list of second coordinates of points inside the file 
            as [y0,...,yi,,..., yn] 
        .zpoints returns the list of third coordinates of points inside the file 
            as [z0,...,zi,,..., zn] 
        .namepoints returns the list of the points name written in the file
            as [name 0, ..., name i, ... name n]
        .nbpoints returns the number of points in the list    
    """
    
    def __init__(self,name,listpoints=[],openingTag= '<begin_xyz>',closingTag= '<end_xyz>'):
        self.name=name
        self.listpoints, self.xpoints, self.ypoints, self.zpoints, self.namepoints, self.nbpoints =self.read()
        self.openingTag= '<begin_xyz>'
        self.closingTag= '<end_xyz>'
        
    def read(self):
        print 'self.name', self.name
        fileInput =open (self.name,'r')
        try :
            file=fileInput.readlines()
        finally :
            fileInput.close
        line=file
        listpoints =[]
        goodfile=0
        nbpoints=0
        xpoints=[]
        ypoints=[]
        zpoints=[]
        namepoints=[]
        for i in range(0,len(file)):
            line[i]=file[i].strip('/n')
            word = line[i].split()
            if word[0]== '<begin_xyz>':
                goodfile=1
            if goodfile==1 and word[0]<> '%' and word[0]<> '<begin_xyz>' and word[0]<> '<end_xyz>':
                point=[]
                point.append(float(word[0]))
                point.append(float(word[1]))
                point.append(float(word[2]))
                listpoints.append(point)
                nbpoints=nbpoints+1
                xpoints.append(float(word[0]))
                ypoints.append(float(word[1]))
                zpoints.append(float(word[2]))
                if len(word)>3:
                    namepoints.append(word[3])
                else:
                    namepoints.append('')                
        fileInput.close
        if goodfile==0:
            print '************************************************'
            print '***** ERROR : this file is not a .xyz file *****'
            print '************************************************'
        else:
            return listpoints, xpoints, ypoints, zpoints, nbpoints, namepoints
            
#==========================================================================================================================
class  Polygons(object)  :
# ===================================
# by Ph. MARON, 6th december 2009
# ===================================
    """ Define a class of object associated with a polygon file .xy of MOHID code
        .listxy give the lists of coordinates of points of the different polygons
            as [[[x00,y00,z00],...,[x0i,y0i,z0i],...][[x10,y10,z10],...,[x1i,y1i,z1i],...]]
        .listx give the lists of coordinates of points of the different polygons
            as [[x00, ...x0i, ...],...], [x10,...,x1i,...], ...]
        .listy give the lists of coordinates of points of the different polygons
            as [[y00, ...y0i, ...],...], [y10,...,y1i,...], ...]
        .vertice returns an array containing the lists of coordinates (x,y) of points of ONE
            polygon   as [[x0,y0], ..., [xi,yi], ...] RUN WITH ONE POLYGON ONLY
            property used for masked zone in plot
    """
    
    def __init__(self,name,listpolygons=[], listx=[], listy=[],openingTag= '<beginpolygon>',closingTag= '<endpolygon>'):
        self.name=name
        self.listpolygons,self.listx, self.listy, self.listxy, self.nbpolygons=self.read()
        self.openingTag= '<beginpolygon>'
        self.closingTag= '<endpolygon>'
    
    def read(self):
        fileInput =open (self.name,'r')
        try :
            file=fileInput.readlines()
        finally :
            fileInput.close
        line=file
        listpolygons =[];polygon=[];x=[];y=[]
        listx=[]; listy=[] ;pointxy=[];listxy=[]
        goodfile=0
        for i in range(0,len(file)):
            line[i]=file[i].strip('/n')
            word = line[i].split()
            if word[0]== '<beginpolygon>':
                goodfile=1
            if goodfile==1 and word[0]<> '%' and word[0]<> '<beginpolygon>' : 
                if word[0]<>'<endpolygon>':
                    point=[];ptxy=[]
                    point.append(float(word[0]))
                    point.append(float(word[1]))
                    #point.append(float(word[2]))
                    polygon.append(point)
                    x.append(float(word[0]))
                    y.append(float(word[1]))
                    ptxy.append(float(word[0]))
                    ptxy.append(float(word[1]))
                    pointxy.append(ptxy)
                else:    
                    listpolygons.append(polygon)
                    listx.append(x)
                    listy.append(y)
                    listxy.append(pointxy)
                    polygon=[];x=[];y=[];ptxy=[]
                    i=i+2
        fileInput.close
        if goodfile==0:
            print '************************************************'
            print '***** ERROR : this file is not a .xy file *****'
            print '************************************************'
        else:
            nbpolygons=array(listx).shape[0]
            return listpolygons, listx, listy, listxy, nbpolygons
    
    def __getattr__(self, name):
        if name in self.__dict__:
            return self.__dict__[name]
        else:
            polygonfile = open(self.name,'a')
            # ==== Group Grid
            if name == 'vertice':
                value = self.get_vertice()
                setattr(self, name, value)
                polygonfile.close()
                return value
    def get_vertice(self):
        # print '***************** get_coordpts'
        fileInput =open (self.name,'r')
        try :
            file=fileInput.readlines()
        finally :
            fileInput.close
        line=file
        goodfile=0
        good=0
        verti=[]
        vertic=[]
        for i in range(len(line)):
            line[i]=line[i].strip('/n')
            word = line[i].split()
            if word[0]== '<beginpolygon>':
                good=1
            if word[0]<> '<beginpolygon>' and  word[0]<>'<endpolygon>'and good==1:
                verti.append([float32(word[0]),float32(word[1])])
            if word[0]=='<endpolygon>':
                good=0
                vertic.append(verti)
                verti=[]
        vertice=array(vertic)
        return vertice
                

#========================================================================================================================== 
import time
import datetime
from matplotlib.dates import date2num
class  Timeserie  :
# ===================================
# by Ph. MARON, 6th december 2009
# ===================================
# 2010/02/18 ==> Timeserie completed with velocityu, velociyuv, velocityw,velociymodulus, velocitydirection 
# =============================================================================
    """ 
    Define a class of object associated with a TimeSerie file .srh of MOHID code
        .seriename give the name of timeseries
        .localisationi give the location of the point as line in the associated grid
        .localisationj give the location of the point as column in the associated grid
        .serieinitialdata give a list containing the initial time of the time serie
            in format [YYYY, MM, DD, hh, mm, ss]
        .initialyear give the year of the initial time of the time serie 
        .initialmonth give the month of the initial time of the time serie
        .initialday give the day of the initial time of the time serie
        .initialhour give the hour of the initial time of the time serie
        .initialminute give the minute of the initial time of the time serie
        .initialseconde give the second of the initial time of the time serie
        .timeunits gives the units used in the time serie
        .modeldomain gives the name of the simulation which obtains this file 
        .titles gives a list of the titles of the different column of the time serie
        .timeserie gives a list of list of results
        .residual  gives a list of the residual values
        .instant returns a list of the different recorded instants in format YYYY-MM-DD HH:mm:ss
        .numinstant returns a list of the different recorded instants in numerical format
        .waterlevel returns a list of waterlevel values at different instants
          .velocityu returns a list of velocity U values at different instants
        .velocityv returns a list of velocity V values at different instants
        .velocityw returns a list of velocity W values at different instants
        .velocitymodulus returns a list of velocity modulus values at different instants
        .velocitydirection returns a list of velocity direction values at different instants
    """
    def __init__(self,name):
        self.name=name  
        self.seriename, self.localisationi, self.localisationj, self.serieinitialdata, \
              self.initialyear, self.initialmonth, self.initialday, self.initialhour, \
              self.initialminute, self.initialseconde, self.timeunits, self.modeldomain, \
              self.titles,self.timeserie, self.residual , self.instant, self.numinstant = self.read()
    
    def read(self):
        fileInput =open (self.name,'r')
        try :
           file=fileInput.readlines()
        finally :
            fileInput.close
        flagserie=0; flagresidual=0
        line=file
        # ==== is it a good file ?
        word = line[-1].split()
        word2=line[-6].split()
        if word[0] =='<EndResidual>'or word[0]=='<EndTimeSerie>'or word2[0]=='<EndTimeSerie>':
            seriename='no NAME in this file'
            modeldomain='no MODEL_DOMAIN in this file'
            residual='no RESIDUAL in this file'
            #==== read the file
            for i in range(0,len(file)):
                line[i]=file[i].strip('/n')
                word = line[i].split()
                if len(word) > 0 :        
                # ==== seriename
                        if word[0] == 'NAME':   
                            seriename=str(word[len(word)-1])
                # ==== location of the point as line in the associated grid
                        elif word[0] == 'LOCALIZATION_I':   
                            localisationi=int(word[len(word)-1])
                # ==== location of the point as column in the associated grid
                        elif word[0] == 'LOCALIZATION_J':   
                            localisationj=int(word[len(word)-1])
                # ====list of information on the origin data year, month, day, hour, minute, sec
                        elif word[0] == 'SERIE_INITIAL_DATA':   
                            serieinitialdata=[]
                            serieinitialdata.append( float(word[len(word)-6]))
                            initialyear=float(word[len(word)-6])
                            serieinitialdata.append( float(word[len(word)-5]))
                            initialmonth=float(word[len(word)-5])
                            serieinitialdata.append( float(word[len(word)-4]))
                            initialday=float(word[len(word)-4])
                            serieinitialdata.append( float(word[len(word)-3]))
                            initialhour= float(word[len(word)-3])
                            serieinitialdata.append( float(word[len(word)-2]))
                            initialminute=float(word[len(word)-2])
                            serieinitialdata.append( float(word[len(word)-1]))
                            initialseconde=float(word[len(word)-1])
                # ==== time units
                        elif word[0] == 'TIME_UNITS':   
                            timeunits=str(word[len(word)-1])
                            flagtime=1
                # ==== identification of model domain
                        elif word[0] == 'MODEL_DOMAIN':   
                            modeldomain=str(word[len(word)-1])
                #==== reading the timeserie
                        elif word[0] == '<BeginTimeSerie>':
                            # ==== identification of titles of the differents column
                            titles = line[i-1].split()
                            timeserie=[]
                            instant=[]
                            numinstant=[]
                            waterlevel=[]
                            flagserie=1
                        elif word[0] == '<EndTimeSerie>':  
                            flagserie=0
                        elif flagserie==1 :
                            timeseri=[]
                            for j in range(len(word)):
                                timeseri.append(float(word[j]))
                            timeserie.append(timeseri)
                            #=== reading the instant value
                            instant.append(datetime.datetime(int(float(word[1])),\
                               int(float(word[2])),int(float(word[3])),\
                               int(float(word[4])),int(float(word[5])),\
                               int(float(word[6]))))
                            numinstant.append(date2num((datetime.datetime(int(float(word[1])),\
                               int(float(word[2])),int(float(word[3])),\
                               int(float(word[4])),int(float(word[5])),\
                               int(float(word[6]))))))
                  #==== reading the residuals
                        elif word[0] == '<BeginResidual>':  
                            flagresidual=1
                            residual=[]
                        elif word[0] == '<EndResidual>':  
                            flagresidual=0   
                        elif flagresidual==1 :
                            for j in range(len(word)):
                                residual.append(float(word[j]))
                                    
            fileInput.close
            return seriename, localisationi, localisationj, serieinitialdata, initialyear, \
               initialmonth, initialday, initialhour, initialminute, initialseconde, timeunits, \
               modeldomain, titles,timeserie, residual, instant, numinstant
        # ==== error message bad file
        else:
            print '*********************************************************'
            print '***** ERROR : this file is not a TimeSerieFile .srh *****'
            print '*********************************************************'
    
    def __getattr__(self, name):
        if name in self.__dict__:
            return self.__dict__[name]
        else:
            if name == 'velocityu':
                value = self.get_column('velocity_U')
                setattr(self, name, value)
                return value
            if name == 'velocityv':
                value = self.get_column('velocity_V')
                setattr(self, name, value)
                return value
            if name == 'velocityw':
                value = self.get_column('velocity_W')
                setattr(self, name, value)
                return value
            if name == 'velocitymodulus':
                value = self.get_column('velocity_modulus')
                setattr(self, name, value)
                return value
            if name == 'velocitydirection':
                value = self.get_column('velocity_direction')
                setattr(self, name, value)
                return value
            if name == 'waterlevel':
                value = self.get_column('water_level')
                setattr(self, name, value)
                return value
                
    def get_column(self,arraytitle): 
        fileInput =open (self.name,'r')
        file=fileInput.readlines()
        line=file ; flagserie=0
        for i in range(0,len(file)):
                line[i]=file[i].strip('/n')
                word = line[i].split()
                if len(word) > 0 and word[0] == '<BeginTimeSerie>':
                    # ==== identification of titles of the differents column
                    titles = line[i-1].split()
                    property=[]
                    if arraytitle in titles:
                        colproperty=titles.index(arraytitle)
                        flagserie=1
                elif len(word) > 0 and word[0] == '<EndTimeSerie>': 
                    flagserie=0
                elif flagserie==1 :
                    property.append(float(word[colproperty]))
        fileInput.close
        return property
#==========================================================================================================================
        
#==========================================================================================================================
from math import cos, sin
from numpy import array,  amax
class  Griddata  :
# ===================================
# by Ph. MARON, 16 th december 2009
# ===================================
    """ 
    Define a class of object associated with a GridData fileInput .dat of MOHID code
        .fillvalue give the value associated to keyword FILL_VALUE ( -99 par dfaut)
        .ilbiub give the list associated to keyword ILB_IUB
        .jlbjub give the list associated to keyword JLB_JUB
        .origin give the list associated to keyword ORIGIN
        .gridangle give the value associated to keyword GRID_ANGLE
        .coordtip give the value associated to keyword COORD_TIP
        .latitude give the value associated to keyword LATITUDE
        .longitude give the value associated to keywords LONGITUDE
        .posx give the list of the first coordinate of points locating at lines intersections of the grid
        .posy give the list of the second coordinate of points locating at lines intersections of the grid
        .valz give the list of the values associated to each cell of the grid
        .gridintersect give the list of location and coordinates of points located
            at intersections of the grid
        .gridcenters give the list of location of grid cells, coordinates of the center
            point of each and valz associated to it
        .nbcenters give the number of gridcenter points
        .bathymetry (for hdf5 integration use, only if gridangle=0)
        .gridlatitude (for hdf5 integration use, only if gridangle=0)
        .gridlongitude (for hdf5 integration use, only if gridangle=0)
        .waterpoints (for hdf5 integration use, only if gridangle=0)
        .maxdepth gives the maximum value of bathymery(for hdf5 integration use, only if gridangle=0)
     """
    
    def __init__(self,name):
        self.name=name  
        self.fillvalue,self.ilbiub, self.jlbjub, self.origin, self.gridangle ,self.coordtip,\
           self.latitude, self.longitude , self.posx, self.posy, self.valz,\
           self.gridintersect,self.gridcenters, self.nbcenters ,self.bathymetry,\
           self.gridlatitude, self.gridlongitude, self.waterpoints, self.maxdepth= self.read()
         
    def read(self):
        fileInput =open (self.name,'r')
        try :
           fichier=fileInput.readlines()
        finally :
            fileInput.close
        line=fichier
        # ==== is it a good fileInput ?
        word = line[-1].split()
        if word[0] =='<EndGridData2D>':
            #==== read the fileInput
            for i in range(0,len(fichier)):
                line[i]=fichier[i].strip('/n')
                word = line[i].split()
                if len(word) > 0 :        
                # ==== fill value value
                        if word[0] == 'FILL_VALUE':   
                            fillvalue=float(word[len(word)-1])
                # ==== list of numbers of cells on Y axe
                        if word[0] == 'ILB_IUB':   
                            ilbiub=[]
                            ilbiub.append( int(word[len(word)-2]))
                            ilbiub.append( int(word[len(word)-1]))
                # ==== list of number of cells on X axe
                        if word[0] == 'JLB_JUB':   
                            jlbjub=[]
                            jlbjub.append( int(word[len(word)-2]))
                            jlbjub.append( int(word[len(word)-1]))
                # ====list of coordinates of the origin point
                        if word[0] == 'ORIGIN':   
                            origin=[]
                            origin.append( float(word[len(word)-2]))
                            origin.append( float(word[len(word)-1]))
                # ==== grid angle value
                        if word[0] == 'GRID_ANGLE':   
                            gridangle=float(word[len(word)-1])
                # ==== coordinate tip value
                        if word[0] == 'COORD_TIP':   
                            coordtip=int(word[len(word)-1])
                # ==== latitude value of the origin point
                        if word[0] == 'LATITUDE':   
                            latitude=float(word[len(word)-1])
                # ==== longitude value of the origin point 
                        if word[0] == 'LONGITUDE':   
                            longitude=float(word[len(word)-1])
                #==== list of indice location  and coordinates location of the grid intersection points [[0,0,x0,y0],[1,0,x1,y1],...[i,j,xi,yj]...]
                        if word[0] == '<BeginXX>':  
                            posx=[]
                            for j in range(i+1,i+jlbjub[1]+2):
                                posx.append(float(line[j].split()[0]))            
                        if word[0] == '<BeginYY>':  
                            posy=[]
                            for j in range(i+1,i+ilbiub[1]+2):
                                posy.append(float(line[j].split()[0]))
                        if word[0] == '<EndYY>':
                            gridintersect=[];pi=104348./33215.
                            gridlatitude=[];gridlongitude=[]
                            waterpoints=[]
                            for y in range (0,len(posy)):
                                gridlat=[];gridlon=[];waterpts=[]
                                for x in range (0,len(posx)):
                                    point=[]
                                    point.append(x)
                                    point.append(y)
                                    point.append(origin[0]+posx[x]*cos(gridangle*pi/180.)-posy[y]*sin(gridangle*pi/180.))
                                    point.append(origin[1]+posx[x]*sin(gridangle*pi/180.)+posy[y]*cos(gridangle*pi/180.))
                                    gridintersect.append(point)
                                    gridlat.append(origin[1]+posy[y])
                                    gridlon.append(origin[0]+posx[x])
                                gridlatitude.append(gridlat)
                                gridlongitude.append(gridlon)
                            nbcenters = (len(posx)-1)*(len(posy)-1)
                #==== list of indice location, coordinates location of the grid cells center points and valz value[[0,0,x0,y0,z00],[1,0,x1,y0,z10],...[i,j,xi,yj,zij]...]
                        if word[0] == '<BeginGridData2D>': 
                            j=1
                            valz=[]
                            # print i+1+j,i+1+(ilbiub[1])*(jlbjub[1])
                            # for j in range (i+1+j,i+1+(ilbiub[1])*(jlbjub[1])):
                                # valz.append(float(line[j].split()[0]))
                            word = line[i+j].split()
                            while word[0]!='<EndGridData2D>': 
                                valz.append(float(line[i+j].split()[0]))
                                j=j+1
                                word = line[i+j].split()

            gridcenters=[]
            bathymetry=[]
            n=0
            for y in range (0,ilbiub[1]):
                bathy=[]
                for x in range (0,jlbjub[1]):
                    point=[]
                    point.append(x)
                    point.append(y)
                    point.append((gridintersect[(jlbjub[1]+1)*y+x][2]+gridintersect[(jlbjub[1]+1)*(y+1)+(x+1)][2])/2.)
                    point.append((gridintersect[(jlbjub[1]+1)*y+x][3]+gridintersect[(jlbjub[1]+1)*(y+1)+(x+1)][3])/2.)
                    
                    # point.append(valz[(ilbiub[1])*x+y])
                    point.append(valz[n])
                    n=n+1
                    gridcenters.append(point)
                    bathy.append(valz[(jlbjub[1])*y+x])
                bathymetry.append(bathy)
            maxdepth=amax(array(bathymetry))
            fileInput.close
            return fillvalue, ilbiub, jlbjub, origin, gridangle, coordtip, latitude, longitude , posx, posy, valz,\
           gridintersect, gridcenters, nbcenters, bathymetry,  gridlatitude, gridlongitude, waterpoints,maxdepth
        # ==== error message bad fileInput
        else:
            print '************************************************************'
            print '***** ERROR : this fileInput is not a GridDatafile.dat *****'
            print '************************************************************'

 
#==========================================================================================================================
if __name__=='__main__' :
    
    # print '======================================================================'
    # print ' Test on Mohidfiles.Points'
    # print '======================================================================'
    # namefile='Mohidfiles.Points.xyz'
    # file=Points(namefile)
    # print file.name
    # print file.listpoints
    # print file.xpoints
    # print file.ypoints
    # print file.zpoints
    # print file.nbpoints
    # print file.namepoints 
    # print file.openingTag
    # print file.closingTag    
    # print '======================================================================'
    # print ' End of test on Mohidfileshdf5.Points'
    # print '======================================================================'
    # n=raw_input('Type Enter to continue with Class  Polygons!')
    
    # print '======================================================================'
    # print ' Test on Mohidfiles.Polygons'
    # print '======================================================================'
    # namefile='Mohidfiles.Polygons.xy'
    # file=Polygons(namefile)
    # print file.name
    # print file.listpolygons
    # print file.listx
    # print file.listy
    # print file.listxy
    # print file.nbpolygons
    # print file.openingTag
    # print file.closingTag
    # print file.vertice
    # print file.vertice.shape[0]
    # print file.vertice[0] , array(file.vertice[0]).shape
    # print file.vertice[1] , array(file.vertice[1]).shape
    # print '======================================================================'
    # print ' End of test on Mohidfileshdf5.Polygons'
    # print '======================================================================'
    # n=raw_input('Type Enter to continue with Class  Timeserie!') 
   
    print '======================================================================'
    print ' Test on Mohidfiles.Timeserie'
    print '======================================================================'
    # print ' not allready verify ======> TO DO '
    namefile='Mohidfiles.Timeserie.dat'
    file=Timeserie(namefile)
    print file.name
    print '======================================================================'
    print ' End of test on Mohidfileshdf5.Timeserie'
    print '======================================================================'
    n=raw_input('Type Enter to continue with Class  Griddata !') 
 
    
    # print '======================================================================'
    # print ' Test on Mohidfiles.Griddata'
    # print '======================================================================'
    # namefile='Mohidfiles.Griddata.dat'
    # file=Griddata(namefile)
    # print 'file.name :', file.name
    # print file.fillvalue
    # print file.ilbiub
    # print file.jlbjub
    # print array(file.bathymetry).shape
    # print file.gridlatitude,  
    # print array(file.gridlatitude).shape
    # print file.origin
    # print file.gridangle
    # print file.coordtip
    # print file.latitude
    # print file.longitude
    # print file.posx
    # print file.posy
    # print len(file.posx), len(file.posy)
    # print file.valz
    # print len(file.valz)
    # print file.gridintersect
    # print file.gridcenters
    # print file.nbcenters
    # print file.bathymetry
    # print file.gridlatitude
    # print file.gridlongitude
    # print file.waterpoints
    # print file.maxdepth
    # print '======================================================================'
    # print ' End of test on Mohidfileshdf5.Griddata'
    # print '======================================================================'
    # n=raw_input('Type Enter to close!')
  
    